<?php

namespace Drupal\xtcelastica\Plugin\XtcHandler;


use Drupal\Component\Serialization\Json;
use Drupal\Core\Site\Settings;
use Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase;
use Drupal\xtc\XtendedContent\API\XtcLoaderServer;
use Elasticsearch\Client;
use Elasticsearch\ClientBuilder;

/**
 * Plugin implementation of the xtc_handler for File.
 *
 */
abstract class ElasticSearchBase extends XtcHandlerPluginBase {

  const TIMEOUT = 5;

  /**
   * @var Client
   */
  protected $client;

  /**
   * @var array
   */
  protected $params;

  /**
   * @var array
   */
  protected $definition;

  /**
   * @var array
   */
  protected $request = [];

  protected function initProcess() {
    $settings = Settings::get('xtc.serve_client')['xtc']['serve_client']['server'];

    $server = XtcLoaderServer::load($this->profile['server']);
    $env = $settings[$this->profile['server']]['env'] ?? $server['env'];

    $protocol = ($server['connection'][$env]['tls']) ? 'https' : 'http';
    $login = (
      !empty($server['connection'][$env]['username']) &&
      !empty($server['connection'][$env]['password'])
    ) ?
      $server['connection'][$env]['username'] . ':' . $server['connection'][$env]['password'] . '@'
      : '';
    $host = $server['connection'][$env]['host'] . ':' . $server['connection'][$env]['port'];
    $hosts[] = $protocol . '://' . $login . $host;

    $params = [
      'timeout' => $this->getTimeout(),
    ];

    $this->client = ClientBuilder::create()
      ->setHosts($hosts)
      ->setConnectionParams($params)
      ->build();

    if(!empty($this->options['index'])){
      $this->params = [
        'index' => $this->options['index'],
        'type' => '_doc',
      ];
    }
  }

  protected function getTimeout() {
    return self::TIMEOUT;
  }

  protected function runProcess() {
    $query = \Drupal::request()->query;
    $qs = $query->all();
    $this->params = array_merge($this->params, $this->options, $qs);
    $this->cleanParams();
  }

  protected function adaptContent() {
  }

  protected function cleanParams(){
  }

}
